Scriptname FGTBountyQuestActor extends ReferenceAlias  

Quest property bountyQuest Auto

ReferenceAlias property bounty Auto

ReferenceAlias property jarl Auto

ReferenceAlias property steward Auto

MiscObject Property gold Auto

GlobalVariable property difficulty Auto

LocationAlias property hold Auto

Keyword property keywordActiveQuest Auto

GlobalVariable property textNotification Auto

Package property packageTemplate Auto

Package markerPackage


Event OnPackageStart(Package akNewPackage)
  Package templete = akNewPackage.GetTemplate()
  if (templete && templete == packageTemplate)
    markerPackage = akNewPackage
  endif
endEvent


Event OnPackageEnd(Package akOldPackage)
  if (markerPackage && akOldPackage == markerPackage)
    if (isRequiredStopQuest())
      stopQuest()
      return
    endif
    Actor a = getActorReference()
    if (isRequiredShutdownQuest(a.getCurrentLocation()))
      shutdownQuest()   
    else
      stopQuest()
    endif
  endif
endEvent


function stopQuest()
  Actor a = getActorRef()
  clear()
  getOwningQuest().stop()
  a.enableAi(false)
  a.enableAi()
endFunction


function shutdownQuest()
  hold.GetLocation().setKeywordData(keywordActiveQuest, 0)
  Actor a = getActorReference()
  a.addItem(gold, (bountyQuest as BQScript).RewardAmount)
  if (textNotification.getValueInt() == 1)
    if (SKSE.GetVersionRelease() > 0)
Debug.notification(a.GetBaseObject().getName() + " has received the reward.")  
    endif
  endif
  bountyQuest.CompleteAllObjectives()
  bountyQuest.stop()
  stopQuest()
endFunction

bool function isRequiredStopQuest()
  return !bountyQuest.isRunning() || bountyQuest.getStage() == 200
endFunction

bool function isRequiredShutdownQuest(Location currentLocation)
  int stage = bountyQuest.getStage()
  if (stage >= 100)
    Actor j = jarl.getActorRef()
    Actor s = steward.getActorRef()
    return (j && currentLocation == j.getCurrentLocation()) || (s && currentLocation == s.getCurrentLocation())
  endif
  return false
endFunction


Event OnLocationChange(Location akOldLoc, Location akNewLoc)
  if (!akNewLoc)
    return
  endIf

  if (isRequiredStopQuest())
    stopQuest()
    return
  elseif (isRequiredShutdownQuest(akNewLoc))
    shutdownQuest()
    return
  endif

  if (bountyQuest.getStage() == 10)
    if (akNewLoc.IsLoaded())
      return
    endif
    Actor boss = bounty.getActorRef()
    if (akNewLoc == boss.getCurrentLocation())
      startBattle()
    endif
  endif
endEvent

function startBattle()
  Actor a = getActorRef()
  Actor boss = bounty.getActorRef()
  if (!a || !boss)
    return
  endif
  if (isWin(a, boss))
    boss.kill()
    int amount = boss.getGoldAmount()
    boss.removeItem(gold, amount, true, getActorReference())
    bountyQuest.setStage(100)
  else
    if (textNotification.getValueInt() == 1)
      if (SKSE.GetVersionRelease() > 0)
Debug.notification(a.getBaseObject().getName() + " defeated at " + boss.getCurrentLocation().getName())
      endif
    endif
    clear()
    getOwningQuest().stop()
    a.enableAi(false)
    a.enableAi()
  endif
endFunction


bool function isWin(Actor a, Actor boss)
  int score = Utility.RandomInt(1, 100)
  if (score == 100)
    return true
  elseif (score == 1)
    return false
  endif
  return score + ((a.getLevel() - boss.getLevel()) / 2) > difficulty.getValueInt()
endFunction


